@@ -1,8 +1,17 @@
 from typing import Any, Dict, List, Optional, Tuple, Union
 
-from pydantic import BaseModel, Extra
+from pydantic import BaseModel as PydanticBaseModel
+from pydantic import Extra, validator
+from segments.exceptions import ValidationError
 from typing_extensions import Literal, TypedDict
 
+
+class BaseModel(PydanticBaseModel):
+    class Config:
+        # https://pydantic-docs.helpmanual.io/usage/model_config/#smart-union
+        smart_union = True
+
+
 #######################################
 # Literals, constants and other types #
 #######################################
@@ -40,13 +49,31 @@ PointcloudAnnotationType = Literal["cuboid"]
 PCDType = Literal["pcd", "kitti", "nuscenes"]
 InputType = Literal["select", "text", "number", "checkbox"]
 Category = Literal[
-    "street_scenery", "garden", "agriculture", "satellite", "people", "medical", "other"
+    "street_scenery",
+    "garden",
+    "agriculture",
+    "satellite",
+    "people",
+    "medical",
+    "fruit",
+    "other",
 ]
+_Category = [
+    "street_scenery",
+    "garden",
+    "agriculture",
+    "satellite",
+    "people",
+    "medical",
+    "fruit",
+    "other",
+]
+DistortionModel = Literal["plumb_bob"]
 RGB = Tuple[int, int, int]
 RGBA = Tuple[int, int, int, int]
 FormatVersion = Union[float, str]
-ObjectAttributes = Dict[str, Union[str, bool]]
-ImageAttributes = Dict[str, Union[str, bool]]
+ObjectAttributes = Dict[str, Optional[Union[str, bool]]]
+ImageAttributes = Dict[str, Optional[Union[str, bool]]]
 
 
 class AuthHeader(TypedDict):
@@ -274,6 +301,7 @@ class Label(BaseModel):
     sample_uuid: str
     label_type: TaskType
     label_status: LabelStatus
+    labelset: str
     attributes: LabelAttributes
     created_at: str
     created_by: str
@@ -319,8 +347,25 @@ class EgoPose(BaseModel):
     heading: XYZW
 
 
+class CameraIntrinsics(BaseModel):
+    intrinsic_matrix: List[List[float]]
+
+
+class CameraExtrinsics(BaseModel):
+    translation: XYZ
+    rotation: XYZW
+
+
+class CalibratedImage(URL):
+    row: int
+    col: int
+    intrinsics: Optional[CameraIntrinsics] = None
+    extrinsics: Optional[CameraExtrinsics] = None
+
+
 class PointcloudSampleAttributes(BaseModel):
     pcd: PCD
+    images: Optional[List[CalibratedImage]] = None
     ego_pose: Optional[EgoPose] = None
     default_z: Optional[float] = None
     name: Optional[str] = None
@@ -353,12 +398,12 @@ class Sample(BaseModel):
     metadata: Dict[str, Any]
     created_at: str
     created_by: str
-    comments: List[str]
+    comments: Optional[List[str]]
     priority: float
-    has_embedding: bool
+    has_embedding: Optional[bool]
     label: Optional[Label]
     issues: Optional[List[Issue]]
-    dataset_full_name: str
+    dataset_full_name: Optional[str]
 
 
 ########################
@@ -422,6 +467,9 @@ class TaskAttributes(BaseModel):
     format_version: Optional[FormatVersion] = None
     categories: Optional[List[TaskAttributeCategory]] = None
 
+    class Config:
+        extra = Extra.allow
+
 
 class Owner(BaseModel):
     username: str
@@ -464,7 +512,7 @@ class Dataset(BaseModel):
     cloned_from: Optional[str] = None
     description: str
     # data_type: DataType
-    category: Category
+    category: str  # Category
     public: bool
     owner: Owner
     created_at: str
@@ -488,6 +536,14 @@ class Dataset(BaseModel):
     # tasks: Optional[List[Dict[str, Any]]] = None
     embeddings_enabled: Optional[bool] = None
 
+    @validator("category")
+    def check_category(cls, category: str) -> str:
+        if category not in _Category and "custom-" not in category:
+            raise ValidationError(
+                f"The category should be one of {_Category}, but is {category}."
+            )
+        return category
+
 
 ####################
 # Segments dataset #
